
----------
*tbd* 2016 V 0.4.0 beta.  Doug Johnson (finson@whidbey.com) 

----------

## DeviceFeature and Device Drivers - Appendix B: Device Registers and Parameter Block Layout

The document device-driver.md describes the overall device feature proposal and most parts of the detailed design.  Appendix A lists the standard virtual registers that all devices can implement.

This Appendix B defines the registers that are actually implemented by several known device drivers and the detailed format of the parameter block bodies they expect.

##StepperDriver

###`CCR::Configure`
####Set stepper motor interface configuration

Configure the stepper motor attached to the given handle and its associated software object.

###`CSR::Configure`
####Get stepper motor interface configuration

Get the configuration of the stepper motor attached to the given handle and its associated software object.

*Method signatures*

`int status(int handle, CSR::Configure, int count, byte *buf)`
`int control(int handle, CCR::Configure, int count, byte *buf)`

*Parameter block buffer*

####2-wire interface (EasyDriver: step and direction)

     0  interface type (0)
     1  steps per revolution (LSB)
     2  steps per revolution (MSB)
     3  direction pin (LSB)
     4  direction pin (MSB)
     5  step pin (LSB)
     6  step pin (MSB)

####2-wire interface (unipolar motor)

     0  interface type (1)
     1  steps per revolution (LSB)
     2  steps per revolution (MSB)
     3  pin1 (LSB)
     4  pin1 (MSB)
     5  pin2 (LSB)
     6  pin2 (MSB)

####4-wire (bipolar motor)

     0  interface type (2)
     1  steps per revolution (LSB)
     2  steps per revolution (MSB)
     3  pin1 (LSB)
     4  pin1 (MSB)
     5  pin2 (LSB)
     6  pin2 (MSB)
     7  pin3 (LSB)
     8  pin3 (MSB)
     9  pin4 (LSB)
    10  pin4 (MSB)

---
###`Stepper::MoveR`
####Move to new relative position

*Method signature*

`int control(int handle, Stepper::MoveR, 5, byte *buf);`

*Parameter block buffer*

     0  delta position (steps) (LSB)
     1  delta position (steps)
     2  delta position (steps)
     3  delta position (steps) (MSB)
     4  block? (0 or 1)

---
###`Stepper::PositionEvent`
####Stopped at most recently commanded position? (synchronous or asynchronous)

*Method signature*

`int status(int handle, Stepper::PositionEvent, 5, byte *buf);`

*Parameter block buffer*

     0  At position? 0 -> no (running), 1 -> yes (stopped)
     1  Position provided?  0 -> no, 1 -> yes
     2  (opt) absolute position (steps) (LSB)
     3  (opt) absolute position (steps)
     4  (opt) absolute position (steps)
     5  (opt) absolute position (steps) (MSB)

---
###`Stepper::RPMSpeeds`
####Set stepping speed using 16-bit Revolutions Per Minute values

###`Stepper::RPMSpeeds`
####Get stepping speed using 16-bit Revolutions Per Minute values

*Method signature*

`int control(int handle, Stepper::RPMSpeeds, 8, byte *buf);`
`int status(int handle, Stepper::RPMSpeeds, 8, byte *buf);`

*Parameter block buffer*

     0  target speed RPM (LSB)
     1  target speed RPM (MSB)
     2  max speed RPM (LSB)
     3  max speed RPM (MSB)
     4  accel (LSB)
     5  accel (MSB)
     6  decel (LSB)
     7  decel (MSB)

##MCP9808 Temperature Sensor

Note that most of the registers on this device are 16-bit (2-byte) and the device expects them to be written in Big Endian order and provides them to a reader in Big Endian order.  So, for reasons of consistency with the Microchip Technology datasheet for the device, the values in the parameter blocks for this device are arranged as Big Endian sequences.

The bytes can be easily put into whichever order is needed by the host using the macros and methods of the ByteOrder class.

###`CDR_Configure`
####Get/set MCP9808 interface configuration

Get/set the configuration of the temperature sensor attached to the given handle and its associated software object.

*Method signatures*

`int status(int handle, CDR_Configure, int count, byte *buf)`
`int control(int handle, CDR_Configure, int count, byte *buf)`

*Parameter block buffer*

     0  sensor config (MSB)
     1  sensor config (LSB)

---
###`UPPER_TEMP`
####Get/set upper temperature limit register.
###`LOWER_TEMP`
####Get/set lower temperature limit register.
###`CRIT_TEMP`
####Get/set critical temperature limit register.

*Method signatures*

`int status(int handle, int reg, 2, byte *buf);`
`int control(int handle, int reg, 2, byte *buf);`

*Parameter block buffer*

     0  temperature °C (MSB)
     1  temperature °C (LSB)

---
###`AMBIENT_TEMP`
####Get ambient temperature in degrees C.

*Method signature*

`int status(int handle, AMBIENT_TEMP, 2, byte *buf);`

*Parameter block buffer*

     0  temperature °C (MSB)
     1  temperature °C (LSB)

---
###`MANUF_ID`
####Get Manufacturer ID
###`DEVICE_ID`
####Get Device ID

*Method signature*

`int status(int handle, MANUF_ID, 2, byte *buf);`
`int status(int handle, DEVICE_ID, 2, byte *buf);`

*Parameter block buffer*

     0  0x00 (manufacturer ID - MSB)
     1  0x54 (manufacturer ID - LSB)

or 

     0  0x04 (device ID - MSB)
     1  0x00 (device ID - LSB)

---
###`RESOLUTION`

####Get/set sensor resolution

*Method signature*

`int status(int handle, RESOLUTION, 1, byte *buf);`
`int control(int handle, RESOLUTION, 1, byte *buf);`

*Parameter block buffer*

     0  0,1,2, or 3 (resolution control bits)

